/////////////////////////////////
// 3D Pong
////
// November 10th, 2006
/////////////////////////////////
// Coder: Slasher & Xigency
/////////////////////////////////

//#define LIGHTS
#define SHOW_FPS


///////////// INCLUDES //////////////

#include <pspkernel.h>
#include <pspdisplay.h>
#include <pspdebug.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <pspctrl.h>
#include <psppower.h>

// Callbacks
#include "callbacks.cpp"

// 3D Stuff
#include <pspge.h>
#include <pspgu.h>
#include <pspgum.h>

// FPS
#include <psprtc.h>

// GFX stuff
#include "graphics.h"
#include "valloc.h"

// Sound
#include "sound.h"

// Physics
#include "physics.h"

// Font
extern "C"
{
	#include "fontloader.h"
}

// Model loading
#include "models/modelLoad.cpp"



// Module info
PSP_MODULE_INFO("3D Pong", 0, 1, 1);
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER);



//////////// DEFINES ///////////////

#define printf			pspDebugScreenPrintf 

#define	SENSITIVITY		1.0f
#define	THRESHHOLD		48

#define	MAP_HEIGHT		6
#define	MAP_WIDTH		5

#define MOVE_SPEED		20.0f
#define COMP_SPEED		20.0f

#define RGB(r, g, b)	(0xFF000000 | ((b)<<16) | ((g)<<8) | (r))

#define THIRD_PERSON	1
#define FIRST_PERSON	2

#define MENU			1
#define	GAME			2
#define	MULTI			3
#define EXIT			4

#define WIN				1
#define	LOSE			2
#define	NORM			3

#define LIGHT_DISTANCE	10.0f

#define CPU_SPEED		333




/////////// STRUCTS ////////////////

struct Vertex __attribute__((aligned(16))) floor_vertices[2*3] =
{
	{1, 0, RGB(0,0,0), 2, 0, 0},
	{0, 0, RGB(0,0,0), 0, 0, 0},
	{1, 1, RGB(0,0,0), 2, 0, 2},

	{0, 1, RGB(0,0,0), 0, 0, 2},
	{1, 1, RGB(0,0,0), 2, 0, 2},
	{0, 0, RGB(0,0,0), 0, 0, 0},
};

struct Vertex __attribute__((aligned(16))) sky_box[12*3] =
{
	{0, 0, RGB(0,0,0),-1,-1, 1},
	{1, 0, RGB(0,0,0),-1, 1, 1},
	{1, 1, RGB(0,0,0), 1, 1, 1},

	{0, 0, RGB(0,0,0),-1,-1, 1},
	{1, 1, RGB(0,0,0), 1, 1, 1},
	{0, 1, RGB(0,0,0), 1,-1, 1},

	{0, 0, RGB(0,0,0),-1,-1,-1},
	{1, 0, RGB(0,0,0), 1,-1,-1},
	{1, 1, RGB(0,0,0), 1, 1,-1},

	{0, 0, RGB(0,0,0),-1,-1,-1},
	{1, 1, RGB(0,0,0), 1, 1,-1},
	{0, 1, RGB(0,0,0),-1, 1,-1},

	{0, 0, RGB(0,0,0), 1,-1,-1},
	{1, 0, RGB(0,0,0), 1,-1, 1},
	{1, 1, RGB(0,0,0), 1, 1, 1},

	{0, 0, RGB(0,0,0), 1,-1,-1},
	{1, 1, RGB(0,0,0), 1, 1, 1},
	{0, 1, RGB(0,0,0), 1, 1,-1},

	{0, 0, RGB(0,0,0),-1,-1,-1},
	{1, 0, RGB(0,0,0),-1, 1,-1},
	{1, 1, RGB(0,0,0),-1, 1, 1},

	{0, 0, RGB(0,0,0),-1,-1,-1},
	{1, 1, RGB(0,0,0),-1, 1, 1},
	{0, 1, RGB(0,0,0),-1,-1, 1},

	{0, 0, RGB(0,0,0),-1, 1,-1},
	{1, 0, RGB(0,0,0), 1, 1,-1},
	{1, 1, RGB(0,0,0), 1, 1, 1},

	{0, 0, RGB(0,0,0),-1, 1,-1},
	{1, 1, RGB(0,0,0), 1, 1, 1},
	{0, 1, RGB(0,0,0),-1, 1, 1},

	{0, 0, RGB(0,0,0),-1,-1,-1},
	{1, 0, RGB(0,0,0),-1,-1, 1},
	{1, 1, RGB(0,0,0), 1,-1, 1},

	{0, 0, RGB(0,0,0),-1,-1,-1},
	{1, 1, RGB(0,0,0), 1,-1, 1},
	{0, 1, RGB(0,0,0), 1,-1,-1},
};

//////////// GLOBALS ///////////////

// Game Variables
float	camX, camY, camZ;
float	camXRot, camYRot, camZRot;

float	curr_ms = 1.0f;

int		cameraState = 1;
int		state = MENU;

int		dispBufferNumber = 0;

SceCtrlData pad, oldpad;

// FPS Counter
u64		last_tick;
u32		tick_res;
int		frame_count = 0;

// Physics
Physics		World;

ColCircle*	CameraObject;
ColCircle*	player1;
ColCircle*	player2;

// Ball vars
float	ballX = 5.0;
float	ballY = 8.0;
float	ballMoveX = 0.06;
float	ballMoveY = 0.06;

// Score
int		p1Score = 0;
int		p2Score = 0;

bool	hit = true;

// Font
Font*	myFont;

// Menu vars
int		select = GAME;
double	rotY = rand()%360;
double	rotSpeed = rand()%1000/20 - 25;


//ColWall*	p1;
//ColWall*	p2;

// Textures
Texture skyTex;
Texture floorTex;
Texture paddleTex;
Texture ballTex;
Texture duckTex;




////////// FUNCTIONS ///////////////

int	initGraphics()
{
	setupGu();
	convertModels();

	// Projection Matrix
 	sceGumMatrixMode(GU_PROJECTION);
	sceGumLoadIdentity();
	sceGumPerspective(65.0f,16.0f/9.0f,0.5f,50.0f);

	// Load textures
	skyTex.load("./resources/sky.png");
	floorTex.load("./resources/grass.png");
	paddleTex.load("./resources/paddle.png");
	duckTex.load("./resources/ducktex.png");
	ballTex.load("./resources/ball.png");
	
	// Vram them
	skyTex.vram();
	floorTex.vram();
	paddleTex.vram();
	duckTex.vram();
	ballTex.vram();

	// Load the font
	myFont = Font_Load("./resources/FONT.TTF");
	Font_SetSize(myFont, 20);

	return 1;
}

void initVars()
{
	switch (cameraState)
	{
		//
		case THIRD_PERSON:

			// Camera Position
			camX = 5.0f;
			camY = 3.0f;
			camZ = 0.0f;

			// Camera Rotation
			camXRot = 30;
			camYRot = 0;
			camZRot = 10;

			break;
		//
		case FIRST_PERSON:

			break;
		//
	}

	hit = true;
	ballX = 5.0;
	ballY = 8.0;
	ballMoveX = 0.06;
	ballMoveY = 0.06;

	state = MENU;
	/*
	player1->x = 5;
	player1->y = 3;
	player2->x = 5;
	player2->y = 15;
	*/

	p1Score = 0;
	p2Score = 0;


	tick_res = sceRtcGetTickResolution();
	sceRtcGetCurrentTick(&last_tick);
}

void initPhysics()
{
	World.LEFT_X = 0;
	World.RIGHT_X = MAP_WIDTH*2;
	World.UPPER_Y = 0;
	World.LOWER_Y = (MAP_HEIGHT*2)+6;

	World.FRICTION = 2.0f;
	World.GRAVITY = 0.0f;
	World.GRAVITY_ANGLE = 0.0f;
	World.LOSS_OF_ENERGY = 0.0f;

	World.SectionWidth = 50;
	World.SectionHeight = 50;
	World.SectionsX = 1;
	World.SectionsY = 1;

	CameraObject = new ColCircle;
	CameraObject->x = camX;
	CameraObject->y = camZ;
	CameraObject->xsp = 0.0f;
	CameraObject->ysp = 0.0f;	
	CameraObject->mass = 10.0f;
	CameraObject->size = 0.85f;
	World.AddCircle( CameraObject );

	player1 = new ColCircle;
	player1->x = 5;
	player1->y = 3;
	player1->xsp = 0.0f;
	player1->ysp = 0.0f;
	player1->mass = 5.0f;
	player1->size = 0.6f;
	World.AddCircle( player1 );

	player2 = new ColCircle;
	player2->x = 5;
	player2->y = 15;
	player2->xsp = 0.0f;
	player2->ysp = 0.0f;
	player2->mass = 5.0f;
	player2->size = 0.6f;
	World.AddCircle( player2 );
}

void pause(int state)
{
	for (;;)
	{
		
		///////////////////
		////// Input //////
		///////////////////
		oldpad = pad;
 		sceCtrlReadBufferPositive(&pad, 1);

		/*
		///////////////////
		////// Render /////
		///////////////////
		// Start the scene
		sceGuStart(GU_DIRECT,list);
		*/

		// A regular pause
		if (state == NORM)
		{
			if (pad.Buttons & PSP_CTRL_CROSS) break;

			// Display
			//Font_PrintScreen(myFont, 100, 100, "Press X to Continue", GU_RGBA(0, 0, 0, 255), (u32*)0x04000000);
		}
		// Winning message pause
		else if (state == WIN)
		{

		}
		// Losing message pause
		else if (state == LOSE)
		{

		}

		/*
		// End the scene
		sceGuFinish();
		sceGuSync(0,0);

		// Flip
		frameBuffer = sceGuSwapBuffers();
		*/
	}
}

int drawDuck(double posX, double posY, double posZ, double rotX, double rotY, double rotZ)
{
	duckTex.assign();

	// Player 1
	sceGumLoadIdentity();
	{
		ScePspFVector3 pos = { posX, posY, posZ };
		ScePspFVector3 rot = { rotX, rotY, rotZ };
		ScePspFVector3 scale = { 0.5f, 0.5f, 0.5f };
		sceGumTranslate(&pos);
		sceGumRotateXYZ(&rot);
		sceGumScale(&scale);
	}
	
	sceGumDrawArray(GU_TRIANGLES,GU_TEXTURE_32BITF|GU_COLOR_8888|GU_VERTEX_32BITF|GU_TRANSFORM_3D,DUCK_TRIANGLES*3,0,duckModel);

	return 1;
}

int drawFloor()
{
	int x, y;

	floorTex.assign();
	
	for (y = 0; y < MAP_HEIGHT; y++)
	{
		for (x = 0; x < MAP_WIDTH; x++)
		{
			sceGumLoadIdentity();
			{
				ScePspFVector3 pos = { x*2, 0, 3+(y*2) };
				ScePspFVector3 rot = { 0, 0, 0 };
				sceGumTranslate(&pos);
				sceGumRotateXYZ(&rot);
			}

			sceGumDrawArray(GU_TRIANGLES,GU_TEXTURE_32BITF|GU_COLOR_8888|GU_VERTEX_32BITF|GU_TRANSFORM_3D,2*3,0,floor_vertices);
		}
	}

	return 1;
}

int drawPaddles()
{
	paddleTex.assign();

	// Player 1
	sceGumLoadIdentity();
	{
		ScePspFVector3 pos = { player1->x, 0.2, player1->y };
		ScePspFVector3 rot = { 0, 0, 0 };
		ScePspFVector3 scale = { 0.5f, 0.2f, 0.1f };
		sceGumTranslate(&pos);
		sceGumRotateXYZ(&rot);
		sceGumScale(&scale);
	}
	
	sceGumDrawArray(GU_TRIANGLES,GU_TEXTURE_32BITF|GU_COLOR_8888|GU_VERTEX_32BITF|GU_TRANSFORM_3D,PADDLE_TRIANGLES*3,0,paddleModel);

	// Comp || Player 2
	sceGumLoadIdentity();
	{
		ScePspFVector3 pos = { player2->x, 0.2, player2->y };
		ScePspFVector3 rot = { 0, 0, 0 };
		ScePspFVector3 scale = { 0.5f, 0.2f, 0.1f };
		sceGumTranslate(&pos);
		sceGumRotateXYZ(&rot);
		sceGumScale(&scale);
	}
	
	sceGumDrawArray(GU_TRIANGLES,GU_TEXTURE_32BITF|GU_COLOR_8888|GU_VERTEX_32BITF|GU_TRANSFORM_3D,PADDLE_TRIANGLES*3,0,paddleModel);

	return 1;
}

int drawBall()
{
	ballTex.assign();

	sceGumLoadIdentity();
	{
		ScePspFVector3 pos = { ballX, 0.3, ballY };
		ScePspFVector3 rot = { 0, 0, 0 };
		ScePspFVector3 scale = { 0.3f, 0.3f, 0.3f };
		sceGumTranslate(&pos);
		sceGumRotateXYZ(&rot);
		sceGumScale(&scale);
	}
	
	sceGumDrawArray(GU_TRIANGLES,GU_TEXTURE_32BITF|GU_COLOR_8888|GU_VERTEX_32BITF|GU_TRANSFORM_3D,BALL_TRIANGLES*3,0,ballModel);

	return 1;
}

int drawSky()
{
	skyTex.assign();

	sceGumLoadIdentity();
	{
		ScePspFVector3 pos = { 5, 0, 6 };
		ScePspFVector3 rot = { 0, 0, 0 };
		ScePspFVector3 scale = { 10.0f, 10.0f, 10.0f };
		sceGumTranslate(&pos);
		sceGumRotateXYZ(&rot);
		sceGumScale(&scale);
	}
	
	sceGumDrawArray(GU_TRIANGLES,GU_TEXTURE_32BITF|GU_COLOR_8888|GU_VERTEX_32BITF|GU_TRANSFORM_3D,12*3,0,sky_box);

	return 1;
}

void menu()
{
	///////////////////
	////// Input //////
	///////////////////
	oldpad = pad;
 	sceCtrlReadBufferPositive(&pad, 1);

	///////////////////
	///// Render //////
	///////////////////

	// Start the scene
	sceGuStart(GU_DIRECT,list);

	// Clear the screen
	sceGuClearColor(RGB(120,140,120));
	sceGuClearDepth(0);
	sceGuClear(GU_COLOR_BUFFER_BIT|GU_DEPTH_BUFFER_BIT);

	// The Camera
	sceGumMatrixMode(GU_VIEW);
	sceGumLoadIdentity();
	{
		float lx = 0, ly = 0, lz = 0;

		camX = CameraObject->x;
		camY = 3.0;
		camZ = CameraObject->y;

		lx = sin(-camYRot*toRads);
		ly = tan(-camXRot*toRads);
		lz = cos(-camYRot*toRads);

		ScePspFVector3 pos = { camX, camY, camZ };
		ScePspFVector3 look = { camX + lx, camY + ly, camZ + lz };
		ScePspFVector3 tilt = { 0.0f, 1.0f, 0.0f };

		sceGumLookAt(&pos,&look,&tilt);
	}

	// Draw all models
	sceGumMatrixMode(GU_MODEL);

	// Sky box
	drawSky();

	// Ducks
	rotY += rotSpeed;
	drawDuck(3.0, 2.0, 4.0, 0, rotY, 0);
	drawDuck(7.0, 2.0, 4.0, 0, rotY, 0);
	drawDuck(8.0, 1.0, 3.0, 0, rotY, 0);
	drawDuck(4.5, 2.0, 1.0, 0, 60*toRads, 0);

	
	// End the scene
	sceGuFinish();
	sceGuSync(0,0);



	///// The Menu /////

	// Input
	if (pad.Buttons != oldpad.Buttons)
	{
		if (pad.Buttons & PSP_CTRL_UP)
		{
			if (select > GAME)
				select--;
			else
				select = EXIT;
		}
		else if (pad.Buttons & PSP_CTRL_DOWN)
		{
			if (select < EXIT)
				select++;
			else
				select = GAME;
		}

		if (pad.Buttons & PSP_CTRL_CROSS)
		{
			// load(GAME);
			state = select;
		}
	}

	// Display
	if (select == GAME)
		Font_PrintScreen(myFont, 165, 40, "Single Player", GU_RGBA(0, 0, 0, 255), (u32*)0x04000000);
	else
		Font_PrintScreen(myFont, 165, 40, "Single Player", GU_RGBA(255, 255, 255, 255), (u32*)0x04000000);

	if (select == MULTI)
		Font_PrintScreen(myFont, 170, 100, "Multiplayer", GU_RGBA(0, 0, 0, 255), (u32*)0x04000000);
	else
		Font_PrintScreen(myFont, 170, 100, "Multiplayer", GU_RGBA(255, 255, 255, 255), (u32*)0x04000000);

	if (select == EXIT)
		Font_PrintScreen(myFont, 202, 160, "Exit", GU_RGBA(0, 0, 0, 255), (u32*)0x04000000);
	else
		Font_PrintScreen(myFont, 202, 160, "Exit", GU_RGBA(255, 255, 255, 255), (u32*)0x04000000);

	Font_PrintScreen(myFont, 310, 200, "Team-Duck", GU_RGBA(0, 0, 0, 255), (u32*)0x04000000);
	


	// Flip
	frameBuffer = sceGuSwapBuffers();

	dispBufferNumber ^= 1;
}

void game()
{
	///////////////////
	////// Input //////
	///////////////////
	sceCtrlSetSamplingCycle(0);
 	sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
	oldpad = pad;
 	sceCtrlReadBufferPositive(&pad, 1);
 
	// Ball movement
	ballX = ballX + ballMoveX;
	ballY = ballY + ballMoveY;



	///// Detection /////

	// Walls
	if (ballX + 1.0 > MAP_WIDTH*2 + 0.5)
	{
		// Increase speed of ball
		if (ballMoveX > 0) ballMoveX = ballMoveX + 0.001;
		else if (ballMoveX < 0) ballMoveX = ballMoveX - 0.001;
		if (ballMoveY > 0) ballMoveY = ballMoveY + 0.001;
		else if (ballMoveY < 0) ballMoveY = ballMoveY - 0.001;
		// Reverse ball
		ballMoveX =- ballMoveX;
	}
	if (ballX < 0.5)
	{
		// Increase speed of ball
		if (ballMoveX > 0) ballMoveX = ballMoveX + 0.001;
		else if (ballMoveX < 0) ballMoveX = ballMoveX - 0.001;
		if (ballMoveY > 0) ballMoveY = ballMoveY + 0.001;
		else if (ballMoveY < 0) ballMoveY = ballMoveY - 0.001;
		// Reverse ball
		ballMoveX =- ballMoveX;
	}

	///// Players /////
	// Player 1
	if ((ballY - 0.5f <= player1->y) && (ballY >= player1->y) && (ballX <= player1->x + 0.7f) && (ballX + 0.7f >= player1->x))
	{
		// Increase speed of ball
		if (ballMoveX > 0) ballMoveX = ballMoveX + 0.001;
		else if (ballMoveX < 0) ballMoveX = ballMoveX - 0.001;
		if (ballMoveY > 0) ballMoveY = ballMoveY + 0.001;
		else if (ballMoveY < 0) ballMoveY = ballMoveY - 0.001;

		hit = true;
		ballY = ballY + 0.1;

		// Reverse ball
		ballMoveY =- ballMoveY;
	}
	// Comp
	if ((ballY <= player2->y) && (ballY + 0.5f >= player2->y) && (ballX <= player2->x + 0.7f) && (ballX + 0.7f >= player2->x))
	{
		// Increase speed of ball
		if (ballMoveX > 0) ballMoveX = ballMoveX + 0.001;
		else if (ballMoveX < 0) ballMoveX = ballMoveX - 0.001;
		if (ballMoveY > 0) ballMoveY = ballMoveY + 0.001;
		else if (ballMoveY < 0) ballMoveY = ballMoveY - 0.001;

		hit = false;
		ballY = ballY - 0.1;

		// Reverse ball
		ballMoveY =- ballMoveY;
	}


	///// Past players /////
	// Player 1
	if (ballY < 0.0)
	{
		ballX = 5.0;
		ballY = 8.0;
		ballMoveX = 0.06;
		ballMoveY = -0.06;
		p2Score++;

		hit = true;

		ballMoveY =- ballMoveY;
	}
	// Comp
	else if (ballY > (MAP_HEIGHT*2)+6)
	{
		ballX = 5.0;
		ballY = 8.0;
		ballMoveX = 0.06;
		ballMoveY = 0.06;
		p1Score++;

		hit = false;

		ballMoveY =- ballMoveY;
	}
		
	///// Controls /////
	int analogY = pad.Ly - 128;

	// Move left
	if (pad.Buttons & PSP_CTRL_LEFT)
	{
		CameraObject->xsp += cos(camYRot*toRads) * MOVE_SPEED * curr_ms;
		player1->xsp += cos(camYRot*toRads) * MOVE_SPEED * curr_ms;
	}
	// Move Right
	if (pad.Buttons & PSP_CTRL_RIGHT)
	{
		CameraObject->xsp -= cos(camYRot*toRads) * MOVE_SPEED * curr_ms;
		player1->xsp -= cos(camYRot*toRads) * MOVE_SPEED * curr_ms;
	}
	// Pause
	if (pad.Buttons != oldpad.Buttons)
	{
		if (pad.Buttons & PSP_CTRL_START)
		{
			//pause(NORM);
		}
	}


	// Camera
	if (cameraState == THIRD_PERSON)
	{
		if (fabs(analogY) > THRESHHOLD) camXRot += (analogY * curr_ms * SENSITIVITY) / 4;
		if (camXRot <= -10) camXRot = -10;
		if (camXRot >= 60)	camXRot = 60;
	}


	///// AI /////
	if (hit == true)
	{
		if (player2->x > ballX) player2->xsp -= cos(camYRot*toRads) * COMP_SPEED * curr_ms;
		if (player2->x < ballX) player2->xsp += cos(camYRot*toRads) * COMP_SPEED * curr_ms;
	}
	else
	{
		if (player2->x > MAP_WIDTH) player2->xsp -= cos(camYRot*toRads) * COMP_SPEED * curr_ms;
		if (player2->x < MAP_WIDTH) player2->xsp += cos(camYRot*toRads) * COMP_SPEED * curr_ms;
	}


	///// WINNER/LOSER /////
	if (p1Score > 4)
	{
		initVars();

		//pause(WIN);
	}
	else if (p2Score > 4)
	{
		initVars();

		//pause(LOSE);
	}

		

	/////////////////
	//// PHYSICS ////
	/////////////////

	World.GlobalUpdate( curr_ms );

		
	///////////////////
	///// Render //////
	///////////////////

	// Start the scene
	sceGuStart(GU_DIRECT,list);

	// Clear the screen
	sceGuClearColor(RGB(120,140,120));
	sceGuClearDepth(0);
	sceGuClear(GU_COLOR_BUFFER_BIT|GU_DEPTH_BUFFER_BIT);



	// Drawing ///////////////////////////////////////////

	// Light
#ifdef LIGHTS
	int i;
	for (i = 0; i < 4; i++)
	{
		ScePspFVector3 pos = { 5, 2, 1*i };
		sceGuLight(i,GU_POINTLIGHT,GU_DIFFUSE_AND_SPECULAR,&pos);
		sceGuLightColor(i,GU_DIFFUSE,0xffff0000);
		sceGuLightColor(i,GU_SPECULAR,0xffffffff);
		sceGuLightAtt(i,0.0f,1.0f,0.0f);
	}

	sceGuSpecular(12.0f);
	sceGuAmbient(0x00222222);
#endif
	

	// The Camera
	sceGumMatrixMode(GU_VIEW);
	sceGumLoadIdentity();
	{
		float lx = 0, ly = 0, lz = 0;

		switch (cameraState)
		{
			//
			case THIRD_PERSON:

				camX = CameraObject->x;
				camY = 3.0;
				camZ = CameraObject->y;

				lx = sin(-camYRot*toRads);
				ly = tan(-camXRot*toRads);
				lz = cos(-camYRot*toRads);

				break;
			//
			case FIRST_PERSON:

				break;
			//
		}

		ScePspFVector3 pos = { camX, camY, camZ };
		ScePspFVector3 look = { camX + lx, camY + ly, camZ + lz };
		ScePspFVector3 tilt = { 0.0f, 1.0f, 0.0f };

		sceGumLookAt(&pos,&look,&tilt);
	}



	// Draw all models
	sceGumMatrixMode(GU_MODEL);

	
	// The Floor
	drawFloor();

	// The sky
	//drawSky();

	// The Paddles
	drawPaddles();

	// The ball
	drawBall();


	// End the scene
	sceGuFinish();
	sceGuSync(0,0);


	// Score
	char s1[100];
	sprintf(s1, "P1 Score: %d", p1Score);
	char s2[100];
	sprintf(s2, "P2 Score: %d", p2Score);
	
	Font_PrintScreen(myFont, 10, 18, s1, GU_RGBA(255, 255, 255, 255), (u32*)0x04000000);
	Font_PrintScreen(myFont, 10, 38, s2, GU_RGBA(255, 255, 255, 255), (u32*)0x04000000);
	//(dispBufferNumber == 1) ? (u32*)0x04000000 : (u32*)0x04000000 + FRAME_SIZE / sizeof(u32)
	


	// Flip
	//sceDisplayWaitVblankStart();
	frameBuffer = sceGuSwapBuffers();

	dispBufferNumber ^= 1;
}


///////////// MAIN FUNCTION /////////////////
int main()
{
	// Initiate
	SetupCallbacks();
	pspDebugScreenInit();

	scePowerSetClockFrequency(CPU_SPEED, CPU_SPEED, CPU_SPEED/2);

	initGraphics();
	initVars();
	initPhysics();


	World.Initialize();


	// Main loop //
	for (;;)
	{
		switch (state)
		{
			//
			case MENU:

				menu();

				break;
			//
			case GAME:

				game();

				break;
			//
			case MULTI:

				state = MENU;

				break;
			//
			case EXIT:

				goto terminate;

				break;
			//
		}

		// FPS
		frame_count++;
 		u64 curr_tick;
		sceRtcGetCurrentTick(&curr_tick);
		float curr_fps = 1.0f / curr_ms;
#ifdef SHOW_FPS
		pspDebugScreenSetXY(0,0);
		printf("fps: %d.%03d ",(int)curr_fps,(int)((curr_fps - (int)curr_fps) * 1000.0f));
#endif
		double time_span = (curr_tick - last_tick) / (float)tick_res;
		curr_ms = time_span / frame_count;
		frame_count = 0;
		sceRtcGetCurrentTick(&last_tick);
	}

terminate:
	scePowerSetClockFrequency(222, 222, 111);
	sceKernelExitGame();
	return 0;
}
